//  Sur un Arduino avec ATmega328, on travaille en bus CAN via le bus SPI
//  Il faut donc le déclarer
#include <SPI.h>

//  Déclaration de la bubliothèque CAN pour ATmega328
#include <mcp_can.h>

const int SPI_CS_PIN = 10;               //  On n'a pas le choix de la broche : c'est la broche 10
MCP_CAN CAN(SPI_CS_PIN);                 //  déclaration de la broche Chip Select du MCP 2515

struct Octet                             //  Le structure Octet servira lors de la gestion de la carte 4 Servos
{
    boolean presence;
    byte    etat;
    boolean anomalie_fin_de_course_trigo;
    boolean anomalie_fin_de_course_horaire;
    boolean fin_de_course_trigo;
    boolean fin_de_course_horaire;
}
  Table_Octet [] =
{
    {false, 0, false, false, false, false},
    {false, 0, false, false, false, false},
    {false, 0, false, false, false, false},
    {false, 0, false, false, false, false},
};

int mesure[4];                           //  Mesure de la tension sur la broche
int nombre_mesures[4];                   //  Nombre de mesures
int tension_moyenne[4];                  //  Tension moyenne mesurée
int tension_finale[4];                   //  Tension finale suite aux mesures
int tension[4];                          //  Tension courante
boolean valid[4];                        //  Validation

byte data[4];                            //  Les données envoyées
byte ancienne_data[4];                   //  Anciennes données envoyées

byte buf[4];                             //  Les données reçues
byte ancien_buf[4];                      //  Anciennes données reçues

int  canId;                              //  Identifiant CAN (sur 11 bits)
byte len = 0;                            //  Nombre d(octets transmis

//-------------------------- Up_down --------------------------------
void up_down (byte ordre, byte pin_bouton)
{
    mesure[ordre]        = analogRead(ordre + pin_bouton);
    nombre_mesures[ordre] = nombre_mesures[ordre] + 1;
    
    if (((tension_moyenne[ordre] > 750) || (tension_moyenne[ordre] < 250)) && (nombre_mesures[ordre] > 2))
    {
        tension_finale[ordre]   = tension_moyenne[ordre];
        nombre_mesures[ordre] = 0;
        tension[ordre]        = 0;
        valid[ordre]          = true;
    }
    else
    {
        tension[ordre]        = tension[ordre] + mesure[ordre];
        tension_moyenne[ordre] = tension[ordre] / nombre_mesures[ordre];    
    }
}

//------------------------------- SETUP ----------------------------------------
void setup()
{
    Serial.begin(115200);

    //  pinMode(1,OUTPUT);  //  LED aiguille 1 vers la gauche. Servira plus tard
                            //  La pin 2 est la pin d'interruption du MCP 2515 et gérée par la bibliothèque
                            //  Donc, pas de déclaration dans le Setup
    pinMode(3,OUTPUT);      //  LED aiguille 1 vers la droite
    
    pinMode(4,OUTPUT);      //  LED aiguille 2 vers la gauche
    pinMode(5,OUTPUT);      //  LED aiguille 2 vers la droite
    
    pinMode(6,OUTPUT);      //  LED aiguille 3 vers la droite
    pinMode(7,OUTPUT);      //  LED aiguille 3 vers la gauche
    
    pinMode(8,OUTPUT);      //  LED aiguille 4 vers la gauche
    pinMode(9,OUTPUT);      //  LED aiguille 4 vers la droite
    
    //  A4 BP présence aiguille 1
    //  A5 BP présence aiguille 2
    //  A6 BP présence aiguille 3 (uniquement analogique)
    //  A7 BP présence aiguille 4 (uniquement analogique)   

START_INIT:

    if(CAN_OK == CAN.begin(CAN_250KBPS))                   // init can bus : baudrate = 250k
    {
        Serial.println("CAN BUS Shield init ok!");
    }
    else
    {
        Serial.println("CAN BUS Shield init fail");
        Serial.println("Init CAN BUS Shield again");
        delay(100);
        goto START_INIT;
    }
}

//------------------------------------ LOOP ------------------------------------------------------------
void loop()
{    
    //---------------- Envoi des données présence aiguille au gestionnaire --------------
    for (byte i = 0; i<4; i++)
    {
        valid[i] = false;  
        up_down(i,18);
        
        if (valid[i] = true)
        {        
            if (tension_finale[i] < 750)
            {
                Table_Octet[i].presence = true;
                data[i] = 0x80;
            }
            else
            {
                Table_Octet[i].presence = false;
                data[i] = 0;
            }          
        }
        valid[i] = false;
    }   

    for (byte i = 0; i<4; i++)
    {
        if (data[i] != ancienne_data[i])
        {
            Serial.println(i);
            Serial.print("Ancienne data : ");
            Serial.print(ancienne_data[i],BIN);
            Serial.print("     Nouvelle data : ");
            Serial.println(data[i],BIN);
            
            ancienne_data[i] = data[i];
            valid[i] = true;
            
            Serial.println(i);
            Serial.print("  2   Ancienne data : ");
            Serial.print(ancienne_data[i],BIN);
            Serial.print("     Nouvelle data : ");
            Serial.println(data[i],BIN);
        }
    }

    //  On n'envoie les infos dans le bus CAN que quand elles ont changé
    //  Ici, on envoie 4 octets :
    //  Le 1er octet concerne l'aiguille 1, le 2ème l'aiguille 2 ... et le 4 ème l'aiguuille 4
    //  L'info d'occupation est dans le bit de poids le plus fort de chaque octet
    
    if ((valid[0] == true)
     || (valid[1] == true)
     || (valid[2] == true)
     || (valid[3] == true))
    {
        //  send data:  id = 0x301, trame satandard, 4 octets, data
        CAN.sendMsgBuf(0x301, 0, 4, data);
    }

    //---------------------------- Réception position des aiguilles ------------------

    if(CAN_MSGAVAIL == CAN.checkReceive()) //  Teste l'arrivée de données sur le bus CAN
    {        
        CAN.readMsgBuf(&len, buf);         //  On extrait des données le nombre d'octet et les données elle mêmes
        
        canId = CAN.getCanId();            //  On extrait l'identifiant 11 bits
     
        //----------------------- Allumage des LED de position des aiguilles (simulation des envois aux bobines) ----------------------
        if(canId == 0x321)
        {
            if ((ancien_buf[0] != buf[0])
             || (ancien_buf[1] != buf[1])
             || (ancien_buf[2] != buf[2])
             || (ancien_buf[3] != buf[3]))
            {
                Serial.println("-----------------------------");
                Serial.println("get data from ID: ");
                Serial.println(canId,HEX);
                Serial.println(len);
    
                for(byte i = 0; i<len; i++)    //  Impression des données
                {
                    Serial.print(buf[i]);
                    Serial.print("\t");
                    Serial.println(buf[i],BIN);
                    ancien_buf[i] = buf[i];
                }
                Serial.println();
                if (bitRead(buf[0],7) == 1)
                {
                    digitalWrite (8,HIGH);
                }
                else
                {
                    digitalWrite (8,LOW);
                }
                if (bitRead(buf[1],7) == 1)
                {
                    digitalWrite (7,HIGH);
                }
                else
                {
                    digitalWrite (7,LOW);
                }
                if (bitRead(buf[2],7) == 1)
                {
                    digitalWrite (4,HIGH);
                }
                else
                {
                    digitalWrite (4,LOW);
                }
                if (bitRead(buf[3],7) == 1)
                {
                    digitalWrite (1,HIGH);
                }
                else
                {
                    digitalWrite (1,LOW);
                }
                if (bitRead(buf[0],6) == 1)
                {
                    digitalWrite (9,HIGH);
                }
                else
                {
                    digitalWrite (9,LOW);
                }
                if (bitRead(buf[1],6) == 1)
                {
                    digitalWrite (6,HIGH);
                }
                else
                {
                    digitalWrite (6,LOW);
                }
                if (bitRead(buf[2],6) == 1)
                {
                    digitalWrite (5,HIGH);
                }
                else
                {
                    digitalWrite (5,LOW);
                }
                if (bitRead(buf[3],6) == 1)
                {
                    digitalWrite (3,HIGH);
                }
                else
                {
                    digitalWrite (3,LOW);
                }                            
            }
        }       
    }
}
