//  Sur un Arduino DUE, on travaille directement sur le bus CAN
//  On ne passe pas par un MCP2515, ni par le bus SPI
#include <due_can.h>

//  Si on travaille avec le port USB, on annule la ligne
//  Si on travaille avec le port natif, on laisse la ligne
//#define Serial SerialUSB

byte ancien_buf301[4];                      //  Les données reçues de l'identifiant 0x301
                                            //  (= BP présence aiguille depuis la carte aiguille)
byte buf301[4];                             //  Les anciennes données reçues

byte ancien_buf201[8];                      //  Les données reçues de l'identifiant 0x201 (= BP position d'aiguille)
                                            //  (= BP position de l'aiguille depuis la carte TCO)
byte buf201[8];                             //  Les anciennes données reçues

byte compteur_aiguille[4];                 
boolean droite[4];

boolean valid_aiguille;
boolean valid_TCO;

byte position_aiguille[4];
byte LED_presence_TCO[4];
byte numero_aiguille;

//  Déclaration du nom des trames :
//  incoming        = trame entrante
//  output_TCO      = trame sortante vers le module TCO
//  output_aiguille = trame sortante vers le module aiguille

CAN_FRAME incoming, output_TCO, output_aiguille;

//---------------------------- SETUP ----------------------------------------------
void setup()
{
    Serial.begin(115200);
    
    //  Initialisation du CAN 0
    Can0.begin(CAN_BPS_250K);
    
    //  Déclaration des filtres. Ici, on laisse tout passer
    //  Identifiants standards en 11 bits
    
    int filter;
    
    for (int filter = 0; filter < 7; filter++)
    {
      	Can0.setRXFilter(filter, 0, 0, false);
      	Can1.setRXFilter(filter, 0, 0, false);
    }   
}

//-------------------------------- Impression du contenu d'une trame -------------------
void printFrame(CAN_FRAME &frame)
{
    Serial.print("ID: 0x");
    Serial.print(frame.id, HEX);
    Serial.print(" Len: ");
    Serial.print(frame.length);
    Serial.print(" Data: 0x");
    for (int count = 0; count < frame.length; count++)
    {
        Serial.print(frame.data.byte[count], HEX);
        Serial.print(" ");
    }
    Serial.print("\r\n");
}
//--------------------------- Transformation 2^n -> n -------------------
byte exposant(byte entree)
{
    switch (entree)
    {
        case 1:
            numero_aiguille = 0;
            break;
        case 2:
            numero_aiguille = 1;
            break;
        case 4:
            numero_aiguille = 2;
            break;
        case 8:
            numero_aiguille = 3;
            break;
    }
    return numero_aiguille;
}
//---------------------------------- LOOP ----------------------------------
void loop()
{
    //------------------------ Réception de données (incoming) -------------------------
    //  Ici, on réceptionne et on stocke les données pour les réenvoyer,
    //  traitées par le gestionnaire, vers les modules concernés

    if (Can0.available() > 0)          //  Teste l'arrivée des données sur le bus Can0
    {
        Can0.read(incoming);      //  Lecture des données
        if (incoming.id == 0x301)
        {
            for(byte i = 0; i<incoming.length; i++)
            {
                 buf301[i] = incoming.data.byte[i];
                 //  incoming.length       = lecture du nombre d'octets transmis
                 //  incoming.data.byte[i] = lecture de l'octet de rang [i]
                         
                 Serial.println("-------");
                 Serial.println(i);
                 Serial.println(buf301[i]);
                 Serial.println(incoming.length);
                      
                 if ((ancien_buf301[0] != buf301[0])
                  || (ancien_buf301[1] != buf301[1])
                  || (ancien_buf301[2] != buf301[2])
                  || (ancien_buf301[3] != buf301[3]))
                 {                             
                     printFrame(incoming);
                     ancien_buf301[i] = buf301[i];
                     Serial.println(ancien_buf301[i],BIN);
                     Serial.println();                        
                 }
            }

            //---------------------- Envoi de données vers TCO (output_TCO) -------------------------------
            //  Récupération, par le gestionnaire, des données du buffer 301
            //  Pour le test des présences, le gestionnaire fait boîte aux lettres
            //  et renvoie tout le courrier vers le TCO. Mais il l'a lu avant !
        
            for (byte i = 0; i<4; i++)
            {
                if (buf301[i] != 0)
                {
                    LED_presence_TCO[i] = 0x20;
                }
                else
                {
                    LED_presence_TCO[i] = 0;
                }
            }
        
            output_TCO.id = 0x211;
            output_TCO.length = 4;                
               
            for (byte i = 0; i<4; i++)
            {
                Serial.print("i out = ");
                Serial.print(i);
                Serial.print("  LED presence TCO = ");
                Serial.print(LED_presence_TCO[i]);
                Serial.print("  position aiguille = ");
                Serial.println(position_aiguille[i]);
                output_TCO.data.byte[i] = LED_presence_TCO[i] + position_aiguille[i];
            }
                
            // Prepare transmit ID, data and data length in CAN0 mailbox 0
                  
            //Send out the frame on whichever mailbox is free or queue it for
            //sending when there is an opening.
            Can0.sendFrame(output_TCO);
        }

   
        //-------------------------- Réception depuis TCO ---------------------------------------

        Can0.read(incoming);      //  Lecture des données
        if (incoming.id == 0x201)
        {
            for(byte i = 0; i<incoming.length; i++)
            {
                buf201[i] = incoming.data.byte[i];
                
                if ((ancien_buf201[0] != buf201[0])
                 || (ancien_buf201[1] != buf201[1])
                 || (ancien_buf201[2] != buf201[2])
                 || (ancien_buf201[3] != buf201[3])
                 || (ancien_buf201[1] != buf201[4])
                 || (ancien_buf201[1] != buf201[5])
                 || (ancien_buf201[1] != buf201[6])
                 || (ancien_buf201[1] != buf201[7]))
                {   
                    printFrame(incoming);
                    ancien_buf201[i] = buf201[i];
                    Serial.println(ancien_buf201[i],BIN);
                    Serial.println();                         
                }
            }
        }

        //------------------- Envoi de données vers aiguilles (output_aiguille) --------------------------                
        valid_aiguille = false;
        //  Récupération, par le gestionnaire, des données du buffer 201
        //  Pour la position des aiguilles, c'est plus compliqué :
        //  1°) On a un BP au TCO qui fait bascule. C'est géré par le gestionnaire
        //  2°) Les infos viennent sur 1 octet (bits 7, 6, 5 et 4 de l'octet 1 ici)
        //  et repartent sur 4 octets (bits 7 ou 6 suivant la position de l'aiguille)
        
        if (buf201[0] != 0)  //  On est sur la carte 0x201 qui gère 4 aiguilles 
        {  
            exposant(buf201[0]);                                   

            Serial.print("numero d'aiguille = ");
            Serial.print(numero_aiguille);
            Serial.print("  buf201 = ");
            Serial.print(buf201[0]);
            Serial.print("  compteur_aiguille (test si BP = 1) = ");
            Serial.print(compteur_aiguille[numero_aiguille]);
            Serial.print("  droite = ");
            Serial.println(droite[numero_aiguille]);
                
            if (compteur_aiguille[numero_aiguille] == 0)
            {
                if ((droite[numero_aiguille] == true) || (position_aiguille[numero_aiguille] == 0))
                {
                    position_aiguille[numero_aiguille] = 0x80;
                    droite[numero_aiguille] = true;
                }
                else
                {
                    position_aiguille[numero_aiguille] = 0x40;
                    droite[numero_aiguille] = false; 
                } 
                compteur_aiguille[numero_aiguille] = 1; 
                valid_aiguille = true;
            }
            if (compteur_aiguille[numero_aiguille] == 2)
            {
                if ((droite[numero_aiguille] == true) || (position_aiguille[numero_aiguille] == 0))
                {
                    position_aiguille[numero_aiguille] = 0x40;
                    droite[numero_aiguille] = true;
                }
                else
                {
                    position_aiguille[numero_aiguille] = 0x80;
                    droite[numero_aiguille] = false; 
                }
                compteur_aiguille[numero_aiguille] = 0;
                valid_aiguille = true;
            }
        }
        else
        {
            exposant(buf201[0]);
            
            Serial.print("numero d'aguille = ");
            Serial.print(numero_aiguille);
            Serial.print("  compteur_aiguille (test si BP = 0) = ");
            Serial.print(compteur_aiguille[numero_aiguille]);
            Serial.print("  droite = ");
            Serial.println(droite[numero_aiguille]);
            if (compteur_aiguille[numero_aiguille] == 1)
            {
                compteur_aiguille[numero_aiguille] = 2;
            }
        }
        
        //  Si on a des données à envoyer, on les envoie
        if (valid_aiguille == true)
        {
            output_aiguille.id = 0x321;
            output_aiguille.length = 4;
            Serial.println("----------------");
               
            for (byte i = 0; i<4; i++)
            {
                Serial.print("i out = ");
                Serial.print(i);
                Serial.print("  position aiguille = ");
                Serial.println(position_aiguille[i]);
                    
                output_aiguille.data.byte[i] = position_aiguille[i];
                output_TCO.data.byte[i] = LED_presence_TCO[i] + position_aiguille[i];
            }  
           
            //  Envoi des données vers le module aiguille              
            Can0.sendFrame(output_aiguille);
            
            //  Envoi des données vers le module TCO    
            output_aiguille.id = 0x211;          
            Can0.sendFrame(output_TCO);
    
            //  Envoi des données vers le module aiguilles
            //  Dans le futur, on enverra les infos pour que le NANO de la carte 4 aiguilles
            //  calcule les PWM à envoyer aux servos de sa carte.
        }        
    }
}



